/*
 ============================================================================
 Name        : multi_card.c
 Author      : www.d-logic.net
 Version     : 1.1
 Description : anti-collision-demo console example written in C
 Description : For demonstration purposes works only with classic cards in
 Description :     transport configuration (tries to authenticate with key B
 Description :     value FF:FF:FF:FF:FF:FF).
 Dependency  : uFCoder library, min. ver. 5.0.10
 Dependency  : uFR firmware from ver. 5.0.24
 ============================================================================
 */

#define STR_APP_VERSION "1.1"

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#if __WIN32 || __WIN64
#	include <conio.h>
#elif linux || __linux__ || __APPLE__
#	include "conio_gnu.h"
#endif
#include "uFCoder.h"
#include "anti_collision.h"
//------------------------------------------------------------------------------
uint8_t ThresholdMinLevel = 15;
uint8_t ThresholdCollLevel = 7;
uint8_t RFLevelAmp = 0;
uint8_t RFLevel = 15;
uint8_t RxGain = 7;
//------------------------------------------------------------------------------
int main(void) {
#define CONTENT_BUFF_LEN	2048
#define STR_BUFF_LEN(x)		STR_BUFF_LEN1(x)
#define STR_BUFF_LEN1(x)	#x
	UFR_STATUS status;
	int8_t IsAntiCollEnabled;
	int8_t IsAnyCardSelected;
	uint8_t CardsNumber;
	uint8_t List[4 * 11]; // Max list size is 44 bytes, first byte is always current UidSize
	uint8_t ListSize;
	uint8_t ListOfSizes[4];
	uint8_t ListOfIndexes[4];
	uint8_t index;
	uint8_t CardType;
	int loop = 1;
	int cancel = 0;
	char c;
	uint8_t key[6] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
	uint8_t selected;
	int nitems, block;
	uint8_t data[17];
	uint8_t sak;
	uint8_t uid_len;

	printf( "\nConsole application example anti-collision-demo v" STR_APP_VERSION "\n"
			"Anti-collision-demo mode demonstration using advantages of the\n"
			"ISO-14443-3 anti collision algorithm.\n\n");
	printf( "uFCoder Library version: %s\n"
			"Please wait while trying connect to the uFR device.\n",
			GetDllVersionStr());

	status = ReaderOpen();
	if (status != UFR_OK) {
		printf("Can't connect to the device. Is uFR connected?\n");
		return EXIT_FAILURE;
	}

	usage();
	GetRfAnalogRegistersTypeA(&ThresholdMinLevel, &ThresholdCollLevel, &RFLevelAmp, &RxGain, &RFLevel);

	while (loop) {
		switch (getch()) {
		case '1':
			if (ThresholdMinLevel > 0) {
				--ThresholdMinLevel;
				status = SetRfAnalogRegistersTypeA(ThresholdMinLevel, ThresholdCollLevel, RFLevelAmp, RxGain, RFLevel);
				if (!status)
					printf("ThresholdMinLevel have been successfully changed to %d.\n", ThresholdMinLevel);
				else
					printf("Status is: %s\n", UFR_Status2String(status));
			}
			else
				printf("Error: ThresholdMinLevel have to be greater or equal to 0.\n");
			break;
		case '2':
			if (ThresholdMinLevel < 15) {
				++ThresholdMinLevel;
				status = SetRfAnalogRegistersTypeA(ThresholdMinLevel, ThresholdCollLevel, RFLevelAmp, RxGain, RFLevel);
				if (!status)
					printf("ThresholdMinLevel have been successfully changed to %d.\n", ThresholdMinLevel);
				else
					printf("Status is: %s\n", UFR_Status2String(status));
			}
			else
				printf("Error: ThresholdMinLevel have to be less or equal to 15.\n");
			break;
		case '3':
			if (ThresholdCollLevel > 0) {
				--ThresholdCollLevel;
				status = SetRfAnalogRegistersTypeA(ThresholdMinLevel, ThresholdCollLevel, RFLevelAmp, RxGain, RFLevel);
				if (!status)
					printf("ThresholdCollLevel have been successfully changed to %d.\n", ThresholdCollLevel);
				else
					printf("Status is: %s\n", UFR_Status2String(status));
			}
			else
				printf("Error: ThresholdCollLevel have to be greater or equal to 0.\n");
			break;
		case '4':
			if (ThresholdCollLevel < 7) {
				++ThresholdCollLevel;
				status = SetRfAnalogRegistersTypeA(ThresholdMinLevel, ThresholdCollLevel, RFLevelAmp, RxGain, RFLevel);
				if (!status)
					printf("ThresholdCollLevel have been successfully changed to %d.\n", ThresholdCollLevel);
				else
					printf("Status is: %s\n", UFR_Status2String(status));
			}
			else
				printf("Error: ThresholdCollLevel have to be less or equal to 7.\n");
			break;
		case '5':
			if (RxGain > 0) {
				--RxGain;
				status = SetRfAnalogRegistersTypeA(ThresholdMinLevel, ThresholdCollLevel, RFLevelAmp, RxGain, RFLevel);
				if (!status)
					printf("RxGain have been successfully changed to %d.\n", RxGain);
				else
					printf("Status is: %s\n", UFR_Status2String(status));
			}
			else
				printf("Error: RxGain have to be greater or equal to 0.\n");
			break;
		case '6':
			if (RxGain < 7) {
				++RxGain;
				status = SetRfAnalogRegistersTypeA(ThresholdMinLevel, ThresholdCollLevel, RFLevelAmp, RxGain, RFLevel);
				if (!status)
					printf("RxGain have been successfully changed to %d.\n", RxGain);
				else
					printf("Status is: %s\n", UFR_Status2String(status));
			}
			else
				printf("Error: RxGain have to be less or equal to 7.\n");
			break;
		case '7':
			if (RFLevel > 0) {
				--RFLevel;
				status = SetRfAnalogRegistersTypeA(ThresholdMinLevel, ThresholdCollLevel, RFLevelAmp, RxGain, RFLevel);
				if (!status)
					printf("RFLevel have been successfully changed to %d.\n", RFLevel);
				else
					printf("Status is: %s\n", UFR_Status2String(status));
			}
			else
				printf("Error: RFLevel have to be greater or equal to 0.\n");
			break;
		case '8':
			if (RFLevel < 15) {
				++RFLevel;
				status = SetRfAnalogRegistersTypeA(ThresholdMinLevel, ThresholdCollLevel, RFLevelAmp, RxGain, RFLevel);
				if (!status)
					printf("RFLevel have been successfully changed to %d.\n", RFLevel);
				else
					printf("Status is: %s\n", UFR_Status2String(status));
			}
			else
				printf("Error: RFLevel have to be less or equal to 15.\n");
			break;
		case '9':
			status = GetRfAnalogRegistersTypeA(&ThresholdMinLevel, &ThresholdCollLevel, &RFLevelAmp, &RxGain, &RFLevel);
			if (!status)
				printf("RF analogue registers current setup in the uFR device:\n"
						"ThresholdMinLevel = %d\n"
						"ThresholdCollLevel = %d\n"
						"RFLevelAmp = %d\n"
						"RxGain = %d\n"
						"RFLevel = %d\n", ThresholdMinLevel, ThresholdCollLevel, RFLevelAmp, RxGain, RFLevel);
			else
				printf("Status is: %s\n", UFR_Status2String(status));
			break;
		case '0':
			status = SetRfAnalogRegistersTypeADefault();
			if (!status)
				printf("RF analogue registers have been successfully REVERTED TO DEFAULT VALUES.\n");
			else
				printf("Status is: %s\n", UFR_Status2String(status));
			break;
		case 'I': // Get reader description:
		case 'i':
			printf("%s\n", GetReaderDescription());
			print_ln('=');
			break;
			//..................................................................
		case 'O': // Get software versions:
		case 'o':
			printf("anti-collision-demo console example version: " STR_APP_VERSION "\n");
			printf("FCoder Library version: %s\n", GetDllVersionStr());
			print_ln('=');
			break;
			//..................................................................
		case 'P': // Get ID of the tag in reader field:
		case 'p':
			status = GetCardIdEx(&sak, data, &uid_len);
			if (status != UFR_OK) {
				printf("Status is: %s\n", UFR_Status2String(status));
				print_ln('=');
				break;
			}
			status = GetDlogicCardType(&CardType);
			if (status != UFR_OK) {
				printf("Status is: %s\n", UFR_Status2String(status));
				print_ln('=');
				break;
			}
			printf("Card type: %s, sak = 0x%02X, uid[%d] = ", UFR_DLCardType2String(CardType), sak, uid_len);
			print_hex_ln(data, uid_len, ":");
			print_ln('=');
			break;
			//..................................................................
		case 'M': // Show current device mode:
		case 'm':
			status = GetAntiCollisionStatus(&IsAntiCollEnabled, &IsAnyCardSelected);
			if (!status)
				if (!IsAntiCollEnabled)
					printf("Anti-collision mode is disabled.\n");
				else
				{
					printf("Anti-collision mode is enabled.\n");
					if (IsAnyCardSelected)
						printf("One card is selected.\n");
					else
						printf("There are no card selected.\n");
				}
			else
				printf("Status is: %s\n", UFR_Status2String(status));
			print_ln('=');
			break;
			//..................................................................
		case 'E': // Enable anti collision mode:
		case 'e':
			status = EnableAntiCollision();
			if (!status)
				printf("Anti-collision mode successfully enabled.\n");
			else
				printf("Status is: %s\n", UFR_Status2String(status));
			print_ln('=');
			break;
			//..................................................................
		case 'D': // Disable anti collision mode:
		case 'd':
			status = DisableAntiCollision();
			if (!status)
				printf("Anti-collision mode successfully disabled.\n");
			else
				printf("Status is: %s\n", UFR_Status2String(status));
			print_ln('=');
			CardsNumber = 0;
			break;
			//..................................................................
		case 'L': // List cards in reader field:
		case 'l':
			status = EnumCards(&CardsNumber, &ListSize);
			if (!status)
			{
				printf("There is %d cards in the reader field.\n", CardsNumber);
			}
			else
			{
				printf("Status is: %s\n", UFR_Status2String(status));
				print_ln('=');
				break;
			}
			if (CardsNumber > 0) {
				status = ListCards(List, ListSize);
				if (!status)
				{
					printf("Uid list of the cards in the reader field are:\n");
					index = 0;
					for (int i = 0; i < CardsNumber; i++) {
						ListOfSizes[i] = List[index];
						ListOfIndexes[i] = index + 1;
						printf("%d. ", i + 1);
						print_hex_ln(&List[ListOfIndexes[i]], ListOfSizes[i], ":");
						index += 11;
					}
				}
				else
				{
					printf("Status is: %s\n", UFR_Status2String(status));
					print_ln('.');
					print_hex_ln(List, ListSize, ":");
					print_ln('=');
					break;
				}
			}
			print_ln('=');
			break;
			//..................................................................
		case 'S': // Select listed card:
		case 's':
			if (!CardsNumber)
			{
				printf("Error: no card enumerated. Enumerate cards first.\n");
				print_ln('=');
				break;
			}
			printf("Choose card uid:\n");
			for (int i = 0; i < CardsNumber; i++) {
				printf("  [%d] ", i + 1);
				print_hex_ln(&List[ListOfIndexes[i]], ListOfSizes[i], ":");
			}
			printf("[esc] Cancel operation.\n");
			fflush(stdin);
			do {
				c = getch();
				switch (c) {
					case '1':
					case '2':
					case '3':
					case '4':
						selected = c - 0x30; // to int
						if (selected > CardsNumber)
							break;
						status = SelectCard(&List[ListOfIndexes[selected - 1]], ListOfSizes[selected - 1], &CardType);
						if (!status)
							printf("Card successfully selected. Selected card type is %s\n", UFR_DLCardType2String(CardType));
						else
							printf("Status is: %s\n", UFR_Status2String(status));
						cancel = 1;
						break;
					case '\x1b': // cancel:
						cancel = 1;
						break;
				}
			} while (!cancel);
			fflush(stdin);
			print_ln('=');
			break;
			//..................................................................
		case 'X': // Deselect card:
		case 'x':
			status = DeslectCard();
			if (!status)
				printf("Card successfully deselected.\n");
			else
				printf("Status is: %s\n", UFR_Status2String(status));
			print_ln('=');
			break;
			//..................................................................
		case 'R': // Block Read:
		case 'r':
			printf("    Enter block address: ");
			nitems = scanf("%d", &block);
			fflush(stdin);
			if ((nitems == EOF) || (nitems == 0)) {
				printf("\nInput error.\n");
				print_ln('=');
				break;
			}
			if ((block < 0) || (block > 0xFF)) {
				printf("\nBlock out of range.\n");
				print_ln('=');
				break;
			}
			status = BlockRead_PK(data, block, MIFARE_AUTHENT1A, key);
			if (!status)
			{
				data[16] = 0;
				printf("Block successfully read. Content is [ASCII]: %s\n", data);
			}
			else
				printf("Status is: %s\n", UFR_Status2String(status));
			print_ln('=');
			break;
			//..................................................................
		case 'W': // Block Write:
		case 'w':
			printf("    Enter block address: ");
			nitems = scanf("%d", &block);
			fflush(stdin);
			if ((nitems == EOF) || (nitems == 0)) {
				printf("\nInput error.\n");
				print_ln('=');
				break;
			}
			if ((block < 0) || (block > 0xFF)) {
				printf("\nPage out of range.\n");
				print_ln('=');
				break;
			}
			printf("    Enter block content [16 ASCII characters]: ");
			uint8_t *temp;
			temp = malloc(1024);
			temp[0] = 0;
			nitems = scanf("%s", temp);
			fflush(stdin);
			if ((nitems == EOF) || (nitems == 0) || strlen((char *)temp) != 16) {
				printf("\nInput error.\n");
				print_ln('=');
				free(temp);
				break;
			}
			status = BlockWrite_PK(temp, block, MIFARE_AUTHENT1B, key);
			if (!status)
				printf("Block successfully written.\n");
			else
				printf("Status is: %s\n", UFR_Status2String(status));
			print_ln('=');
			free(temp);
			break;
			//..................................................................
		case 'H': // Show help screen:
		case 'h':
			usage();
			break;
			//..................................................................
		case '\x1b': // escape application:
			status = ReaderClose();
			if (status != UFR_OK) {
				printf("Error while closing device.\n");
				return EXIT_FAILURE;
			}
			loop = 0;
			break;
		}
	}

	return EXIT_SUCCESS;
}
//------------------------------------------------------------------------------
void usage(void) {
	print_ln('=');
	printf("\nCommand keys (case insensitive):\n");
	print_ln('-');

	printf( "Options specific for uFR devices with RF Booster:\n"
			"[1/2] ThresholdMinLevel:  [1] Decrease / [2] Increase\n"
			"[3/4] ThresholdCollLevel: [3] Decrease / [4] Increase\n"
			"[5/6] RxGain:             [5] Decrease / [6] Increase\n"
			"[7/8] RFLevel:            [7] Decrease / [8] Increase\n"
			"  [9] Get current setup of the RF analogue registers from the uFR device\n"
			"  [0] Set RF analogue registers to default values\n"
			"-----------------------------------------------------\n\n"
			"Options for all uFR devices:\n"

			"  [I] Get reader description.\n"
			"  [O] Get software versions.\n"
			"  [P] Get ID of the selected tag.\n"

			"  [M] Show current device mode.\n"
			"  [E] Enable anti collision mode.\n"
			"  [D] Disable anti collision mode.\n"

			"  [L] List cards in reader field.\n"
			"  [S] Select listed card.\n"
			"  [X] Deselect card.\n"

			"  [R] Block Read.\n"
			"  [W] Block Write.\n"

			"  [H] This help screen.\n"
			"[esc] Close device and exit program.\n");
	print_ln('=');
}
//------------------------------------------------------------------------------
void Printout(void) {

	printf("Set:\n"
			"     [0] for 0\n");
	printf(
			"     [1] for 1\n");
	print_ln('-');
	printf("Enter your choice:\n");
}
//------------------------------------------------------------------------------
void SetMode(void) {
	fflush(stdin);
}
//------------------------------------------------------------------------------
void print_ln_len(char symbol, uint8_t cnt) {

	for (int i = 0; i < cnt; i++)
		printf("%c", symbol);

	printf("\n");
}
//------------------------------------------------------------------------------
void print_ln(char symbol) {

	print_ln_len(symbol, DEFAULT_LINE_LEN);
}
//------------------------------------------------------------------------------
void print_hex(const uint8_t *data, uint32_t len, const char *delimiter) {

	for (int i = 0; i < len; i++) {
		printf("%02X", data[i]);
		if ((delimiter != NULL) && (i < (len - 1)))
			printf("%c", *delimiter);
	}
}
//------------------------------------------------------------------------------
void print_hex_ln(const uint8_t *data, uint32_t len, const char *delimiter) {

	print_hex(data, len, delimiter);
	printf("\n");
}
//==============================================================================
